/*
    Requiered librarys to have in place:
    ArduinoJson 6.x
    CRC32 V2.0.0 by Christopher Baker 
    PubSubClient V2.7 or newer by Nick O'Leary <nick.oleary@gmail.com>
    TFT_eSPI 1.4.9 by Bodmer

    As addition URLs in the Arduino IDE use:
    https://dl.espressif.com/dl/package_esp32_dev_index.json
    https://github.com/Bodmer/TFT_eSPI
    https://github.com/littlevgl/lv_arduino

    This shall give you access to all the requiered libs
    
    Also not to configure the littlevgl end TFT_eSPI!
    In the library folder for the littlvgl edit lv_config.h or replace it with the suppiled file in this ZIP this sketch came with
    Fo the TFT_eSPI edit the User_Setup.h for your needs or used the supplied files that shipped with this ZIP
    

*/


#include <lvgl.h>
#include <TFT_eSPI.h> 
#include <Ticker.h>

#include <WiFi.h>
#include <WiFiClient.h>
#include <esp_wifi.h>
#include <FS.h>
#include <SPIFFS.h>

#include <PubSubClient.h>                   // MQTT-Library
#include "ArduinoJson.h"

#include "HMI.h"
#include "datastore.h"




/**********************************
 *      DEFINES
 **********************************/
#define LVGL_TICK_PERIOD 20
/**********************************
 *  GLOBAL VARIABLES / OBJECTS
 **********************************/
TFT_eSPI tft = TFT_eSPI();      /* generate tft instance */ 
Ticker   lvgl_tick;             /* timer for lvgl */

//Memory for the displaybuffer
static lv_disp_buf_t disp_buf;
static lv_color_t buf[LV_HOR_RES_MAX * 10];

TaskHandle_t Task1;
TaskHandle_t MQTTTaskHandle;
SemaphoreHandle_t xUIMutex=NULL;

/**************************************************************************************************
 *    Function      : disp_flush_data
 *    Description   : Sends pixels to the display
 *    Input         : lv_disp_drv_t *disp, int32_t x1, int32_t y1, int32_t x2, int32_t y2, const lv_color_t *color_array 
 *    Output        : none
 *    Remarks       : none
 **************************************************************************************************/
void disp_flush_data(lv_disp_drv_t *disp, const lv_area_t *area, lv_color_t *color_p)
{
  uint16_t c; //used to transfer a current pixel
  /* We set swapped bytes to ture, otherwyse the color get wrong */
  tft.setSwapBytes(true);
  tft.startWrite(); /* Start new TFT transaction */
  tft.setAddrWindow(area->x1, area->y1, (area->x2 - area->x1 + 1), (area->y2 - area->y1 + 1)); /* set the working window */
  for (int y = area->y1; y <= area->y2; y++) {
    for (int x = area->x1; x <= area->x2; x++) {
      c = color_p->full;
      tft.writeColor(c, 1);
      color_p++;
    }
  }
  tft.endWrite(); /* terminate TFT transaction */
  /*Tell the flushing is ready*/
  lv_disp_flush_ready(disp); /* tell lvgl that flushing is done */
  
}

/**************************************************************************************************
 *    Function      : x2064_tp_read
 *    Description   : reads the actual status of the touchscreen
 *    Input         : lv_indev_data_t *data 
 *    Output        : none
 *    Remarks       : We don't use the TP_IRQ pin 
 **************************************************************************************************/
bool x2064_tp_read(lv_indev_drv_t * indev, lv_indev_data_t * data)
{
  /* we store the last x and y coodinates here */
  static uint16_t last_x=0;
  static uint16_t last_y=0;
  
  uint16_t x=4096;
  uint16_t y=4096;
  /* 
   *  if the dirver would requiere a second call to get all data 
   *  this must be as long true as we need to collet it. As we only
   *  have a touchscreen this gets pretty simple
   */
  bool moredata = false; /* We get all data in once call */
  /*Check if we have given not a NULL-pointer */
  assert(data);
  /* 
   *  This collects the raw data from the screen 
   *  to save one pin we ignore the TP_IRQ pin 
   */
  tft.getTouchRaw(&x,&y);
  /* 
   *  We need to convert the raw data to screen coordinates
   */
  tft.convertRawXY(&x,&y);

  if( (x<=480) && (y<=320) ){
    /* Depending on the screenrotation we need to shift coordinates */
    if(3 == tft.getRotation() ){
      data->point.x = 480-x;
      data->point.y = 320-y;
    } else if ( 1 == tft.getRotation() ){
      data->point.x = x;
      data->point.y = y;
    } else {
      /* The current config will not support potrait mode */
      assert(NULL);
    }
    last_x=x;
    last_y=y;
    data->state = LV_INDEV_STATE_PR;  

  } else {
    data->state = LV_INDEV_STATE_REL; 
    data->point.x = last_x;
    data->point.y = last_y; 
   
  }
  return moredata;      
}

/**************************************************************************************************
 *    Function      : hal_init
 *    Description   : Sets up the dirver and hooks for littlevgl
 *    Input         : none 
 *    Output        : none
 *    Remarks       : none
 **************************************************************************************************/
void hal_init(void)
{
   lv_disp_buf_init(&disp_buf, buf, NULL, LV_HOR_RES_MAX * 10);

    /*Initialize the display*/
    lv_disp_drv_t disp_drv;
    lv_disp_drv_init(&disp_drv);
    disp_drv.hor_res = 480;
    disp_drv.ver_res = 320;
    disp_drv.flush_cb = disp_flush_data;
    disp_drv.buffer = &disp_buf;
    lv_disp_drv_register(&disp_drv);
    
    /*Initialize the touch screen*/
    lv_indev_drv_t indev_drv;
    lv_indev_drv_init(&indev_drv); 
    indev_drv.type = LV_INDEV_TYPE_POINTER;
    indev_drv.read_cb = x2064_tp_read;
    lv_indev_drv_register(&indev_drv);


}

/**************************************************************************************************
 *    Function      : lv_tick_handler
 *    Description   : requiered to be called periodically by a ticker 
 *    Input         : none 
 *    Output        : none
 *    Remarks       : none
 **************************************************************************************************/
void lv_tick_handler() {
  lv_tick_inc(LVGL_TICK_PERIOD);
}



/**************************************************************************************************
 *    Function      : setup
 *    Description   : Startupfunction for the Arduinoframworc
 *    Input         : none 
 *    Output        : none
 *    Remarks       : Only called once
 **************************************************************************************************/
void setup() {
 
   tft.init();
   /* If we set Rotation to 3 we also need to invert X and Y */
   tft.setRotation(3);
   /* Init fir the lvgl library */
   lv_init();
   /* init for the screen and touch driver objects */
   hal_init();
   /* Setup the ticker to have the lvgl get it's timing */
   lvgl_tick.attach_ms(LVGL_TICK_PERIOD, lv_tick_handler);

   lv_theme_t * th = lv_theme_zen_init(210, &lv_font_roboto_22);     //Set a HUE value and a Font for the Night Theme
   SetupHMI( th );
   /* We now setup the rest of the requiered system */

   
   Serial.begin(115200);
   datastoresetup();
   /* This is for the flash file system to access the webcontent */
   SPIFFS.begin();
   xUIMutex = xSemaphoreCreateMutex();
   Serial.println(F("Init WiFi"));  
   initWiFi();

   xTaskCreatePinnedToCore(
   MQTT_Task,
   "MQTT_Task",
   10000,
   NULL,
   1,
   &MQTTTaskHandle,
   1);
}


/**************************************************************************************************
 *    Function      : loop
 *    Description   : Superloop, called as fast as possible
 *    Input         : none 
 *    Output        : none
 *    Remarks       : This is done in a low priority task
 **************************************************************************************************/
void loop() {

  /* 
   *  If no weatherstation is arround we can use the demo mode with some random data
   *  set use_demo to ture for this purpose
   */
  bool use_demo = false;

  
  static uint32_t last_millis=0;
  static uint8_t val=0;
  static int8_t temp=-127;
  static uint16_t pressure = 850;
  static uint16_t winddir=0;
  static float windspeed=0;

  static float current_rain=0;
  static float avgone_rain=0;
  static float avgeight_rain=0;
  
  /* handler to keep the UI going */
  if (true == xSemaphoreTake(xUIMutex ,1)){
    lv_task_handler();
    xSemaphoreGive(xUIMutex);
  }
  NetworkTask();
  // This is for pure demon purpose 
  if(true == use_demo ){
    if( ( last_millis + 250 ) < millis() ){
      last_millis=millis();
      UpdateHumidity(val);
      if(val<100){
        val++;
      } else {
        val =0;
      }
      UpdateTemperature(temp);
      temp++;
  
      UpdateAirPressure( pressure );
      pressure++;
      if(pressure>1110){
        pressure = 840;
      }
  
      winddir++;
      if(winddir>720){
        winddir=0;
      }
      UpdateWindDirection(winddir);
  
      windspeed+=0.335;
      if(windspeed > 200 ){
        windspeed=0;
      }
      UpdateWindSpeed(windspeed);
  
      current_rain+=0.1;
      if(current_rain>400){
        current_rain=0;
      }
  
      avgone_rain+=0.01;
      if(avgone_rain>400){
        avgone_rain=0;
      }
      
      avgeight_rain+=0.03;
      if(avgeight_rain>150){
        avgeight_rain=0;
      }
  
      UpdateRainChart( current_rain );
      UpdateRainCurrent( current_rain );
      UpdateRainAvgOneHour( avgone_rain );
      UpdateRainAvgEightHours( avgeight_rain );
  
      
     } 
  }

   
   
}


void MQTT_Task( void* prarm ){
 
   uint32_t ulNotificationValue;
   int32_t last_message = millis();
   uint32_t last_recontry=0;
   mqttsettings_t Settings = eepread_mqttsettings();
                         
   Serial.println("MQTT Thread Start");
   WiFiClient espClient;                       // WiFi ESP Client  
   PubSubClient client(espClient);             // MQTT Client 
   client.setCallback(callback);             // define Callback function
   while(1==1){

   /* if settings have changed we need to inform this task that a reload and reconnect is requiered */ 
   ulNotificationValue = ulTaskNotifyTake( pdTRUE, 0 );

   if( ulNotificationValue&0x01 != 0 ){
      Serial.println("Reload MQTT Settings");
      /* we need to reload the settings and do a reconnect */
      if(true == client.connected() ){
        client.disconnect();
      }
      Settings = eepread_mqttsettings();
   }

   if(!client.connected()) {             
        /* sainity check */
        if( (Settings.mqttserverport!=0) && (Settings.mqttservename[0]!=0) && ( Settings.enable != false ) ){

              if( (last_recontry+10000) < millis() ){
                last_recontry = millis();
                Serial.print("Connecting to MQTT...");  // connect to MQTT
                client.setServer(Settings.mqttservename, Settings.mqttserverport); // Init MQTT     
                if (client.connect(Settings.mqtthostname, Settings.mqttusername, Settings.mqttpassword)) {
                  Serial.println("connected");          // successfull connected  
                  client.subscribe(Settings.mqtttopic);             // subscibe MQTT Topic
                } else {
                  Serial.println("failed");   // MQTT not connected       
                }
              }
        }
   } else{
        client.loop();                            // loop on client
   }
   delay(10);
 }
}

/***************************
 * callback - MQTT message
 ***************************/
void callback(char* topic, byte* payload, unsigned int length) {

 if(length < 4096 ){ /* we only process messages up to 4096 elements */
  //Serial.print("Message arrived in topic: ");  // print "MQTT-Message arrived"
  //Serial.print(topic);
 
  DynamicJsonDocument doc(4096);
  deserializeJson(doc, payload);
  
  JsonObject data = doc["data"];
  
  float data_wind_direction = data["wind"]["direction"]; // 255.1
  float data_wind_speed = data["wind"]["speed"]; // 1.54
  
  int data_rain = data["rain"]; // 0
  float data_temperature = data["temperature"]; // 22.1
  float data_humidity = data["humidity"]; // 38.4
  float data_airpressure = data["airpressure"]; // 998.4
  int data_PM2_5 = data["PM2_5"]; // 0
  int data_PM10 = data["PM10"]; // 0
  
  float station_battery = doc["station"]["battery"]; // 12.84
  bool station_charging = doc["station"]["charging"]; // false
  // We need to lock access to he UI as these is not thread safe 
  if (true == xSemaphoreTake(xUIMutex ,portMAX_DELAY)){
    UpdateHumidity( (uint8_t)data_humidity );
    UpdateTemperature( (int16_t)data_temperature ); /* Limited to -128°C to +128°C */
    UpdateAirPressure( (uint16_t)data_airpressure ); /* Should not be negative */
    UpdateWindSpeed( data_wind_speed );
    UpdateWindDirection( (uint16_t)data_wind_direction);
    
    //UpdateRainChart( float latest_value);
    UpdateRainCurrent( data_rain);
    //UpdateRainAvgOneHour( float latest_value);
    //UpdateRainAvgEightHours( float latest_value);
    //RefreshRainChart( float lastvalue_array[7] );
    
    xSemaphoreGive(xUIMutex);
  }
   /* We need to update the elements to the ui from our main thread */
  

 }
  
}
