#include <stdio.h>
#include "HMI.h"

/* We need some pointer definded here to update the values */
static lv_obj_t * humidity_lmeter;
static lv_obj_t * humidity_label;

static lv_obj_t * temp_lmeter;
static lv_obj_t * temp_label;
static lv_obj_t * degree_label;



static lv_obj_t * air_pressure_gauge;

static lv_obj_t * winddir;
static lv_obj_t * windspeed_lmeter;

static lv_obj_t * windspeed;

static lv_obj_t * avg_eighthours;
static lv_obj_t * avg_onehour;
static lv_obj_t * current_value;
static lv_obj_t * chart;
static lv_chart_series_t * ser1;

void SetupBarometerPage( lv_obj_t * parent );
void SetupWindPage( lv_obj_t * parent );
void SetupRainPage( lv_obj_t* parent );

void SetupHMI( lv_theme_t * th ){
    /* We work on a theamed based */
    lv_theme_set_current(th);
  
    /* Next is to create a screen */
    lv_obj_t * scr = lv_cont_create(NULL, NULL);
    lv_scr_load(scr);

    /* And now to populate our four tabs */
    lv_obj_t * tv = lv_tabview_create(scr, NULL);
    lv_obj_set_size(tv, LV_HOR_RES_MAX, LV_VER_RES_MAX);
    lv_obj_t * tab0 = lv_tabview_add_tab(tv, "Barometer");
    lv_obj_t * tab1 = lv_tabview_add_tab(tv, "Wind");
    lv_obj_t * tab2 = lv_tabview_add_tab(tv, "Rain");

    /* 
     *  As for now the Tabs are just empty ones so we need to attach
     *  some elements to it 
     */

    SetupBarometerPage( tab0 ) ;
    SetupWindPage( tab1 );
    SetupRainPage ( tab2 );


}

void SetupBarometerPage( lv_obj_t * parent ){
    


/*********************************
 * Create a greater line meter
 *********************************/
lv_obj_t * label_name;
lv_obj_t * temp_label_name;
lv_obj_t * air_pressure_label_name;
/*Create a new style*/
static lv_style_t style_lmeter2;
lv_style_copy(&style_lmeter2, &lv_style_pretty_color);
style_lmeter2.line.width = 2;
style_lmeter2.line.color = LV_COLOR_SILVER;
style_lmeter2.body.padding.top = 8;
style_lmeter2.body.padding.bottom = 8;
style_lmeter2.body.padding.left = 8;
style_lmeter2.body.padding.right = 8;

style_lmeter2.body.main_color = LV_COLOR_LIME;
style_lmeter2.body.grad_color = LV_COLOR_RED;

/*Create the line meter*/
humidity_lmeter = lv_lmeter_create(parent, NULL);
lv_obj_set_style(humidity_lmeter, &style_lmeter2);
lv_obj_set_size(humidity_lmeter, 120, 120);
lv_obj_align(humidity_lmeter, NULL, LV_ALIGN_IN_TOP_LEFT, 0, 20);
lv_lmeter_set_scale(humidity_lmeter, 240, 31);
lv_lmeter_set_value(humidity_lmeter, 90);

/*Create a label style with smaller font*/
static lv_style_t style_degree_label;
lv_style_copy(&style_degree_label, &lv_style_pretty);
style_degree_label.text.font = &lv_font_roboto_16;
style_degree_label.text.color = LV_COLOR_GRAY;

/*Create a label style with greater font*/
static lv_style_t style_label;
lv_style_copy(&style_label, &lv_style_pretty);
style_label.text.font = &lv_font_roboto_22;
style_label.text.color = LV_COLOR_GRAY;


/*Add a label to show the current value*/
humidity_label = lv_label_create(humidity_lmeter, NULL);
lv_label_set_text(humidity_label, "N/A");
lv_obj_set_style(humidity_label, &style_label);
lv_obj_align(humidity_label, NULL, LV_ALIGN_CENTER, 0, 0);


/*Create a label style with greater font*/
static lv_style_t style_labelname;
lv_style_copy(&style_labelname, &lv_style_pretty);
style_labelname.text.font = &lv_font_roboto_28;
style_labelname.text.color = LV_COLOR_GRAY;
/*Add a label to show the current name*/
label_name = lv_label_create(parent, NULL);
lv_label_set_text(label_name, "Humidity");
lv_obj_set_style(label_name, &style_labelname);
lv_obj_align(label_name, humidity_label, LV_ALIGN_OUT_BOTTOM_MID, 0, 20);


/* Next is the linemeter for the Temperature */
static lv_style_t style_lmeter3;
lv_style_copy(&style_lmeter3, &lv_style_pretty_color);
style_lmeter3.line.width = 2;
style_lmeter3.line.color = LV_COLOR_SILVER;
style_lmeter3.body.padding.top = 8;
style_lmeter3.body.padding.bottom = 8;
style_lmeter3.body.padding.left = 8;
style_lmeter3.body.padding.right = 8;


style_lmeter3.body.main_color = LV_COLOR_NAVY;
style_lmeter3.body.grad_color = LV_COLOR_RED;

/*Create the line meter*/
temp_lmeter = lv_lmeter_create(parent, NULL);
lv_obj_set_style(temp_lmeter, &style_lmeter3);
lv_obj_set_size(temp_lmeter, 160, 160);
lv_obj_align(temp_lmeter, NULL, LV_ALIGN_IN_TOP_MID, -10, 45);
lv_lmeter_set_scale(temp_lmeter, 240, 40);
/* We need to map -50°C to +50°C as 0% to 100% */
lv_lmeter_set_value(temp_lmeter, 50);



/*Add a label to show the current value*/
temp_label = lv_label_create(temp_lmeter, NULL);
lv_label_set_text(temp_label, "0 C");
lv_obj_set_style(temp_label, &style_label);
lv_obj_align(temp_label, NULL, LV_ALIGN_CENTER, -10, 0);

/*Add a label to show the current value*/
degree_label = lv_label_create(temp_lmeter, NULL);
lv_label_set_text(degree_label, "o");
lv_obj_set_style(degree_label, &style_degree_label);
lv_obj_align(degree_label, temp_label, LV_ALIGN_IN_RIGHT_MID, -24, -6);




/*Add a label to show the current name*/
temp_label_name = lv_label_create(parent, NULL);
lv_label_set_text(temp_label_name, "Temperature");
lv_obj_set_style(temp_label_name, &style_labelname);
lv_obj_align(temp_label_name, temp_lmeter, LV_ALIGN_OUT_BOTTOM_MID, 0, -28);

/* Last one in this screen is the air pressure gauge */

/*Create a style*/
static lv_style_t gstyle;
lv_style_copy(&gstyle, &lv_style_pretty_color);
gstyle.body.main_color = lv_color_hex3(0x666);     /*Line color at the beginning*/
gstyle.body.grad_color =  lv_color_hex3(0x666);    /*Line color at the end*/
gstyle.body.padding.left= 8;                      /*Scale line length*/
gstyle.body.padding.top = 4;                       /*Center needle size*/
gstyle.body.padding.bottom = 4;                       /*Center needle size*/
gstyle.body.padding.right = 4;                       /*Center needle size*/
gstyle.body.padding.inner = 8 ;                    /*Scale label padding*/
gstyle.body.border.color = lv_color_hex3(0x333);   /*Needle middle circle color*/
gstyle.line.width = 2;
gstyle.text.color = lv_color_hex3(0x333);
gstyle.line.color = LV_COLOR_RED;                  /*Line color after the critical value*/
gstyle.text.font = &lv_font_roboto_22;
gstyle.text.color = LV_COLOR_GRAY;


/*Describe the color for the needles*/
static lv_color_t needle_colors[] = {LV_COLOR_GRAY};

/*Create a gauge*/
air_pressure_gauge = lv_gauge_create(parent, NULL);
lv_gauge_set_style(air_pressure_gauge,LV_GAUGE_STYLE_MAIN, &gstyle);
lv_gauge_set_needle_count(air_pressure_gauge, 1, needle_colors);
lv_obj_set_size(air_pressure_gauge, 135, 135);
lv_obj_align(air_pressure_gauge, NULL, LV_ALIGN_IN_TOP_RIGHT, -10, 25);
lv_gauge_set_scale(air_pressure_gauge, 250, 40, 3); 
lv_gauge_set_range(air_pressure_gauge, 850, 1100);
lv_gauge_set_critical_value(air_pressure_gauge, 1030);
/*Set the values*/
lv_gauge_set_value(air_pressure_gauge, 0, 984);

/*Add a label to show the current name*/
air_pressure_label_name = lv_label_create(parent, NULL);
lv_label_set_text(air_pressure_label_name, "Pressure");
lv_obj_set_style(air_pressure_label_name, &style_labelname);
lv_obj_align(air_pressure_label_name, air_pressure_gauge, LV_ALIGN_OUT_BOTTOM_MID, 0, -28);


lv_page_set_sb_mode(parent, LV_SB_MODE_OFF );
}




void UpdateHumidity( uint8_t value ){
char buffer[12]={0,};
lv_lmeter_set_value(humidity_lmeter, value);
snprintf ( buffer, sizeof(buffer),"%i%%",value);
lv_label_set_text(humidity_label, buffer);

  
}

void UpdateTemperature( int16_t value ){ /* Limited to -128°C to +128°C */

char buffer[12]={0,};
snprintf ( buffer, sizeof(buffer),"%i  C",value);
lv_label_set_text(temp_label, buffer);

  if(value<=-50){
    lv_lmeter_set_value(temp_lmeter, 0);
  } else if(value>=50){
    lv_lmeter_set_value(temp_lmeter, 100);
  } else {
    lv_lmeter_set_value(temp_lmeter,(value+50) );
  }

  lv_label_set_text(degree_label, "o");
  lv_obj_align(degree_label, temp_label, LV_ALIGN_IN_RIGHT_MID, -14, -6);

}

void UpdateAirPressure( uint16_t value ){ /* Should not be negative */
  /* Check the limits we have */
  if(lv_gauge_get_min_value(air_pressure_gauge)>value){
    lv_gauge_set_value(air_pressure_gauge,0, lv_gauge_get_min_value(air_pressure_gauge));
  } else if (lv_gauge_get_max_value(air_pressure_gauge)<value ) {
    lv_gauge_set_value(air_pressure_gauge,0, lv_gauge_get_max_value(air_pressure_gauge));
  } else {
    lv_gauge_set_value(air_pressure_gauge, 0, value);
  }
}


void SetupWindPage( lv_obj_t * parent ){

/*Create a style*/
static lv_style_t style;
lv_style_copy(&style, &lv_style_pretty_color);
style.body.main_color = lv_color_hex3(0x666);     /*Line color at the beginning*/
style.body.grad_color =  lv_color_hex3(0x666);    /*Line color at the end*/

style.body.padding.top = 10;
style.body.padding.bottom = 10;
style.body.padding.left = 8;
style.body.padding.right = 8;


style.body.padding.inner = 8 ;                    /*Scale label padding*/
style.body.border.color = lv_color_hex3(0x333);   /*Needle middle circle color*/
style.line.width = 3;
style.text.color = lv_color_hex3(0x333);
style.line.color = LV_COLOR_RED;                  /*Line color after the critical value*/


/*Describe the color for the needles*/
static lv_color_t needle_colors[] = { LV_COLOR_PURPLE };


/*Create a label style with greater font*/
static lv_style_t style_labelname;
lv_style_copy(&style_labelname, &lv_style_pretty);
style_labelname.text.font = &lv_font_roboto_22;
style_labelname.text.color = LV_COLOR_GRAY;
  

/*Create a gauge*/

winddir = lv_gauge_create(parent, NULL);
lv_gauge_set_style(winddir, LV_GAUGE_STYLE_MAIN, &style);
lv_gauge_set_needle_count(winddir, 1, needle_colors);
lv_obj_set_size(winddir, 160 , 160);
lv_obj_align(winddir, NULL, LV_ALIGN_IN_TOP_LEFT, 10, 10);
lv_gauge_set_scale(winddir, 359, 30, 0); 
lv_gauge_set_range(winddir, 0, 359);
lv_gauge_set_critical_value(winddir, 361);
lv_gauge_set_value(winddir, 0, 0);

/*Add a label to show the current name*/
lv_obj_t * winddir_label_n = lv_label_create(winddir, NULL);
lv_label_set_text(winddir_label_n, "N");
lv_obj_set_style(winddir_label_n, &style_labelname);
lv_obj_align(winddir_label_n, NULL, LV_ALIGN_CENTER, 0, -50);

/*Add a label to show the current name*/
lv_obj_t * winddir_label_o = lv_label_create(winddir, NULL);
lv_label_set_text(winddir_label_o, "O");
lv_obj_set_style(winddir_label_o, &style_labelname);
lv_obj_align(winddir_label_o, NULL, LV_ALIGN_CENTER, 50, 0);

/*Add a label to show the current name*/

lv_obj_t * winddir_label_s = lv_label_create(winddir, NULL);
lv_label_set_text(winddir_label_s, "S");
lv_obj_set_style(winddir_label_s, &style_labelname);
lv_obj_align(winddir_label_s, NULL, LV_ALIGN_CENTER, 0, 50);

/*Add a label to show the current name*/
lv_obj_t * winddir_label_w = lv_label_create(winddir, NULL);
lv_label_set_text(winddir_label_w, "W");
lv_obj_set_style(winddir_label_w, &style_labelname);
lv_obj_align(winddir_label_w, NULL, LV_ALIGN_CENTER, -50, 0);




/*Create a new style*/
static lv_style_t style_lmeter2;
lv_style_copy(&style_lmeter2, &lv_style_pretty_color);
style_lmeter2.line.width = 2;
style_lmeter2.line.color = LV_COLOR_SILVER;
//style_lmeter2.body.padding.hor = 16;            /*Line length*/
style_lmeter2.body.main_color = LV_COLOR_LIME;
style_lmeter2.body.grad_color = LV_COLOR_RED;

/*Create the line meter*/
windspeed_lmeter = lv_lmeter_create(parent, NULL);
lv_obj_set_style(windspeed_lmeter, &style_lmeter2);
lv_obj_set_size(windspeed_lmeter, 150, 150);
lv_obj_align(windspeed_lmeter, NULL, LV_ALIGN_IN_BOTTOM_RIGHT, -50, -50);
lv_lmeter_set_scale(windspeed_lmeter, 240, 31);
lv_lmeter_set_range(windspeed_lmeter, 0, 120);
lv_lmeter_set_value(windspeed_lmeter, 0);

/*Create a label style with greater font*/
static lv_style_t style_label;
lv_style_copy(&style_label, &lv_style_pretty);
style_label.text.font = &lv_font_roboto_22;
style_label.text.color = LV_COLOR_GRAY;

/*Add a label to show the current value*/
windspeed = lv_label_create(windspeed_lmeter, windspeed);
lv_label_set_text(windspeed, "0 km/h");
lv_obj_set_style(windspeed, &style_label);
lv_obj_align(windspeed, NULL, LV_ALIGN_CENTER, 0, 0);
/*Add a label to show the current name*/
lv_obj_t * windospeed_label_name = lv_label_create(windspeed_lmeter, NULL);
lv_label_set_text(windospeed_label_name, "Windspeed");
lv_obj_set_style(windospeed_label_name, &style_labelname);
lv_obj_align(windospeed_label_name, NULL, LV_ALIGN_IN_BOTTOM_MID, 0, 0);

lv_page_set_sb_mode(parent, LV_SB_MODE_OFF );

}

void UpdateWindSpeed( float speed_kilometer_hour){
  char buffer[12]={0,};
  lv_lmeter_set_value(windspeed_lmeter, speed_kilometer_hour);
  snprintf ( buffer, sizeof(buffer),"%.1fkm/h",speed_kilometer_hour);
  lv_label_set_text(windspeed, buffer);
  lv_obj_align(windspeed, NULL, LV_ALIGN_CENTER, 0, 0);

}

void UpdateWindDirection( uint16_t dir){
  /* We are 180° off for the graphical display */
  dir+=180;
  if(dir>=360){
    dir = dir % 360;
  }
  lv_gauge_set_value(winddir, 0, dir);
  

}

void SetupRainPage( lv_obj_t * parent  ){

  
    /*Create a style for the chart*/
  static lv_style_t style;
  lv_style_copy(&style, &lv_style_pretty);
  style.body.shadow.width = 6;
  style.body.shadow.color = LV_COLOR_GRAY;
  style.line.color = LV_COLOR_GRAY;
  
  /*Create a chart*/



  chart = lv_chart_create(parent, NULL);

  lv_obj_set_size(chart, 200, 150);
  lv_obj_set_style(chart, &style);
  lv_obj_align(chart, NULL, LV_ALIGN_IN_TOP_LEFT, 50, 10);
  lv_chart_set_type(chart, LV_CHART_TYPE_POINT | LV_CHART_TYPE_LINE);   /*Show lines and points too*/
  lv_chart_set_series_opa(chart, LV_OPA_70);                            /*Opacity of the data series*/
  lv_chart_set_series_width(chart, 4);                                  /*Line width and point radious*/
  
  lv_chart_set_range(chart, 0, 400);
  lv_chart_set_point_count(chart, 8);


  lv_chart_set_y_tick_texts(chart, "400\n300\n200\n100\n400", 2, LV_CHART_AXIS_DRAW_LAST_TICK);
  lv_chart_set_y_tick_length(chart, 10, 5);
  lv_chart_set_x_tick_texts(chart,"6h\n4h\n2j\nNow", 2,LV_CHART_AXIS_DRAW_LAST_TICK);
  lv_chart_set_div_line_count(chart, 3, 3);
  lv_chart_set_margin(chart, 50);



  /*Add two data series*/
  ser1 = lv_chart_add_series(chart, LV_COLOR_RED);
  
  /*Directly set points */
  lv_chart_set_next(chart, ser1, 0);
  lv_chart_set_next(chart, ser1, 0);
  lv_chart_set_next(chart, ser1, 0);
  lv_chart_set_next(chart, ser1, 0);
  lv_chart_set_next(chart, ser1, 0);
  lv_chart_set_next(chart, ser1, 0);
  lv_chart_set_next(chart, ser1, 0);
  lv_chart_set_next(chart, ser1, 0);
  


  /* 
   *  Next are three Labels to give the current reading
   *  the avg for one hour and the avg for 8 hours
   */

  /*Create a label style with greater font*/
  static lv_style_t style_values;
  lv_style_copy(&style_values, &lv_style_pretty);
  style_values.text.font = &lv_font_roboto_22;
  style_values.text.color = LV_COLOR_GRAY;
  
    /*Add a label to show the current value*/
  lv_obj_t * current_value_lb = lv_label_create(parent, NULL);
  lv_label_set_text(current_value_lb, "Current");
  lv_obj_set_style(current_value_lb, &style_values);
  lv_obj_align(current_value_lb, NULL, LV_ALIGN_IN_TOP_RIGHT, -30, 0);

   /*Add a label to show the current value*/
  current_value = lv_label_create(parent, NULL);
  lv_label_set_text(current_value, "000.0 l/h");
  lv_obj_set_style(current_value, &style_values);
  lv_obj_align(current_value, NULL, LV_ALIGN_IN_TOP_RIGHT, -10, 35);
  

    /*Add a label to show the avg for one hour*/
  lv_obj_t * avg_onehour_lb = lv_label_create(parent, NULL);
  lv_label_set_text(avg_onehour_lb, "Avg. 1 Hour");
  lv_obj_set_style(avg_onehour_lb, &style_values);
  lv_obj_align(avg_onehour_lb, NULL, LV_ALIGN_IN_RIGHT_MID, -10, 0);

  /*Add a label to show the avg for one hour*/
  avg_onehour= lv_label_create(parent, NULL);
  lv_label_set_text(avg_onehour, "000.0 l/h");
  lv_obj_set_style(avg_onehour, &style_values);
  lv_obj_align(avg_onehour, NULL, LV_ALIGN_IN_RIGHT_MID, -10, 30);


    /*Add a label to show the avg for eight hours*/
  lv_obj_t * avg_eighthours_lb = lv_label_create(parent, NULL);
  lv_label_set_text(avg_eighthours_lb, "Avg. 8 Hours");
  lv_obj_set_style(avg_eighthours_lb, &style_values);
  lv_obj_align(avg_eighthours_lb, NULL, LV_ALIGN_IN_BOTTOM_RIGHT, -10, -30);

      /*Add a label to show the avg for eight hours*/
  avg_eighthours = lv_label_create(parent, NULL);
  lv_label_set_text(avg_eighthours, "000.0 l/h");
  lv_obj_set_style(avg_eighthours, &style_values);
  lv_obj_align(avg_eighthours, NULL, LV_ALIGN_IN_BOTTOM_RIGHT, -10, 0);

  lv_page_set_sb_mode(parent, LV_SB_MODE_OFF );
}

void UpdateRainChart( float latest_value){
   lv_chart_set_next(chart, ser1, latest_value);
}

void RefreshRainChart( float lastvalue_array[7]){
  lv_chart_clear_serie(chart, ser1 );
  for(uint8_t i=0;i<7;i++){
    lv_chart_set_next(chart, ser1, lastvalue_array[i]);
  }
}

void UpdateRainCurrent( float latest_value){
  char buffer[12]={0,};
  snprintf ( buffer, sizeof(buffer),"%.1fl/h",latest_value);
  lv_label_set_text(current_value, buffer);
}

void UpdateRainAvgOneHour( float latest_value){
   char buffer[12]={0,};
   snprintf ( buffer, sizeof(buffer),"%.1fl/h",latest_value);
   lv_label_set_text(avg_onehour, buffer);
}

void UpdateRainAvgEightHours( float latest_value){
  char buffer[12]={0,};
  snprintf ( buffer, sizeof(buffer),"%.1fl/h",latest_value);
  lv_label_set_text(avg_eighthours, buffer);
}
